@@ -179,8 +179,8 @@ class TestUpload:
             pytest.skip("h3 not supported")
         count = 2
         upload_size = 128*1024
-        url = f'https://localhost:{env.https_port}/curltest/put?id=[0-{count-1}]'
-        client = LocalClient(name='hx-upload', env=env)
+        url = f'https://localhost:{env.https_port}/curltest/put'
+        client = LocalClient(name='hx_upload', env=env)
         if not client.exists():
             pytest.skip(f'example client not built: {client.name}')
         r = client.run(args=[
@@ -195,8 +195,8 @@ class TestUpload:
             pytest.skip("h3 not supported")
         count = 2
         upload_size = 128*1024
-        url = f'https://localhost:{env.https_port}/curltest/put?id=[0-{count-1}]'
-        client = LocalClient(name='hx-upload', env=env)
+        url = f'https://localhost:{env.https_port}/curltest/put'
+        client = LocalClient(name='hx_upload', env=env)
         if not client.exists():
             pytest.skip(f'example client not built: {client.name}')
         r = client.run(args=[
@@ -211,8 +211,8 @@ class TestUpload:
             pytest.skip("h3 not supported")
         count = 2
         upload_size = 128*1024
-        url = f'https://localhost:{env.https_port}/curltest/echo?id=[0-{count-1}]'
-        client = LocalClient(name='hx-upload', env=env)
+        url = f'https://localhost:{env.https_port}/curltest/echo'
+        client = LocalClient(name='hx_upload', env=env)
         if not client.exists():
             pytest.skip(f'example client not built: {client.name}')
         r = client.run(args=[
@@ -414,6 +414,8 @@ class TestUpload:
     def test_07_36_upload_30x(self, env: Env, httpd, nghttpx, redir, proto):
         if proto == 'h3' and not env.have_h3():
             pytest.skip("h3 not supported")
+        if proto == 'h3' and env.curl_uses_ossl_quic():
+            pytest.skip("OpenSSL's own QUIC is flaky here")
         if proto == 'h3' and env.curl_uses_lib('msh3'):
             pytest.skip("msh3 fails here")
         data = '0123456789' * 10
@@ -431,6 +433,8 @@ class TestUpload:
     def test_07_37_upload_307(self, env: Env, httpd, nghttpx, proto):
         if proto == 'h3' and not env.have_h3():
             pytest.skip("h3 not supported")
+        if proto == 'h3' and env.curl_uses_ossl_quic():
+            pytest.skip("OpenSSL's own QUIC is flaky here")
         if proto == 'h3' and env.curl_uses_lib('msh3'):
             pytest.skip("msh3 fails here")
         data = '0123456789' * 10
@@ -539,7 +543,7 @@ class TestUpload:
             pytest.skip("h3 not supported")
         if proto == 'h3' and env.curl_uses_lib('msh3'):
             pytest.skip("msh3 fails here")
-        client = LocalClient(name='upload-pausing', env=env, timeout=60)
+        client = LocalClient(name='upload_pausing', env=env, timeout=60)
         if not client.exists():
             pytest.skip(f'example client not built: {client.name}')
         url = f'https://{env.authority_for(env.domain1, proto)}/curltest/echo?id=[0-0]&die_after=0'
@@ -561,10 +565,10 @@ class TestUpload:
             pytest.skip("h3 not supported")
         if proto == 'h3' and env.curl_uses_lib('msh3'):
             pytest.skip("msh3 fails here")
-        client = LocalClient(name='upload-pausing', env=env, timeout=60)
+        client = LocalClient(name='upload_pausing', env=env, timeout=60)
         if not client.exists():
             pytest.skip(f'example client not built: {client.name}')
-        url = f'https://{env.authority_for(env.domain1, proto)}/curltest/echo?id=[0-0]&just_die=1'
+        url = f'https://{env.authority_for(env.domain1, proto)}/curltest/echo?id=0&just_die=1'
         r = client.run(['-V', proto, url])
         exp_code = 52  # GOT_NOTHING
         if proto == 'h2' or proto == 'h3':
@@ -578,10 +582,10 @@ class TestUpload:
             pytest.skip("h3 not supported")
         if proto == 'h3' and env.curl_uses_lib('msh3'):
             pytest.skip("msh3 fails here")
-        client = LocalClient(name='upload-pausing', env=env, timeout=60)
+        client = LocalClient(name='upload_pausing', env=env, timeout=60)
         if not client.exists():
             pytest.skip(f'example client not built: {client.name}')
-        url = f'https://{env.authority_for(env.domain1, proto)}/curltest/echo?id=[0-0]&die_after_100=1'
+        url = f'https://{env.authority_for(env.domain1, proto)}/curltest/echo?id=0&die_after_100=1'
         r = client.run(['-V', proto, url])
         exp_code = 52  # GOT_NOTHING
         if proto == 'h2' or proto == 'h3':
@@ -606,6 +610,30 @@ class TestUpload:
                           extra_args=['--trace-config', 'all'])
         r.check_stats(count=count, http_status=413, exitcode=0)
 
+    @pytest.mark.parametrize("proto", ['http/1.1', 'h2', 'h3'])
+    @pytest.mark.parametrize("httpcode", [301, 302, 307, 308])
+    def test_07_44_put_redir(self, env: Env, httpd, nghttpx, proto, httpcode):
+        if proto == 'h3' and not env.have_h3():
+            pytest.skip("h3 not supported")
+        count = 1
+        upload_size = 128*1024
+        url = f'https://localhost:{env.https_port}/curltest/put-redir-{httpcode}'
+        client = LocalClient(name='hx_upload', env=env)
+        if not client.exists():
+            pytest.skip(f'example client not built: {client.name}')
+        r = client.run(args=[
+             '-n', f'{count}', '-l', '-S', f'{upload_size}', '-V', proto, url
+        ])
+        r.check_exit_code(0)
+        results = [int(m.group(1)) for line in r.trace_lines
+                     if (m := re.match(r'.* FINISHED, result=(\d+), response=(\d+)', line))]
+        httpcodes = [int(m.group(2)) for line in r.trace_lines
+                     if (m := re.match(r'.* FINISHED, result=(\d+), response=(\d+)', line))]
+        if httpcode == 308:
+            assert results[0] == 65, f'{r}'  # could not rewind input
+        else:
+            assert httpcodes[0] == httpcode, f'{r}'
+
     # speed limited on put handler
     @pytest.mark.parametrize("proto", ['http/1.1', 'h2', 'h3'])
     def test_07_50_put_speed_limit(self, env: Env, httpd, nghttpx, proto):
@@ -717,8 +745,8 @@ class TestUpload:
         port = env.port_for(proto)
         if proto != 'h3':
             port = env.nghttpx_https_port
-        url = f'https://{env.domain1}:{port}/curltest/put?id=[0-{count-1}]'
-        client = LocalClient(name='hx-upload', env=env)
+        url = f'https://{env.domain1}:{port}/curltest/put'
+        client = LocalClient(name='hx_upload', env=env)
         if not client.exists():
             pytest.skip(f'example client not built: {client.name}')
         r = client.run(args=[
@@ -740,7 +768,7 @@ class TestUpload:
         assert earlydata[0] == 0, f'{earlydata}\n{r.dump_logs()}'
         # depending on cpu load, curl might not upload as much before
         # the handshake starts and early data stops.
-        assert 102 <= earlydata[1] <= exp_early, f'{earlydata}\n{r.dump_logs()}'
+        assert 0 < earlydata[1] <= exp_early, f'{earlydata}\n{r.dump_logs()}'
 
     def check_downloads(self, client, r, source: List[str], count: int,
                         complete: bool = True):
